# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import curses.panel
import fcntl
import os.path
import struct
import termios
import window
import events
import filelistwin
import playlistwin
import playerwin
import iteminfowin
import helper
import helpwin
import statusbar
# import songdb3 as songdb
import songdb
import players
import timer
import config

from helper import debug

class mainscreen:
    def __init__(self, screen):
        self.screen = screen
        self.h, self.w = screen.getmaxyx()

        leftpanelw = int(self.w/2.2)
        rightpanelw = self.w-leftpanelw

        config.logfile = os.path.expanduser(config.logfile)

        self.channel = events.hub.newchannel()
        self.timer = timer.timer()

        self.songdb = songdb.songdb(self.channel, config)

        self.keybindings = config.keybindings["global"]

        self.statusbar = statusbar.statusbar(screen,
                                             self.h-1, self.w, 
                                             config.colors["statusbar"],
                                             self.channel)

        self.filelistwin = filelistwin.filelistwin(screen,
                                                   self.h-1, leftpanelw,
                                                   0, 0,
                                                   config.colors["filelist"],
                                                   config.filelistwinhasscrollbar,
                                                   self.channel,
                                                   config.basedir,
                                                   self.songdb,
                                                   config.keybindings["filelist"],
                                                   )

        self.playlistwin = playlistwin.playlistwin(screen,
                                                   self.h-6-3-1, rightpanelw,
                                                   9, leftpanelw,
                                                   config.colors["playlist"],
                                                   config.playlistwinhasscrollbar,
                                                   self.channel,
                                                   self.songdb,
                                                   config.logfile,
                                                   config.keybindings["playlist"])

        if config.inputwinpopup:
            from inputwin import popupinputwin as inputwin
        else:
            from inputwin import statusbarinputwin as inputwin

        self.inputwin = inputwin(screen,
                                 self.h, self.w,
                                 config.colors["input"],
                                 self.channel,
                                 config.keybindings["global"])

        self.players = []
        for player in config.players:
            name = player["name"]
            options = player["options"]
            if name=="xmms":
                import players.xmmsplayer
                self.players.append(players.xmmsplayer.player(**options))
            elif name=="mpg123":
                import players.mpg123player
                self.players.append(players.mpg123player.player(**options))
            elif name=="ao":
                import players.aoplayer
                self.players.append(players.aoplayer.player(**options))

        for player in self.players:
            player.start()

        self.playerwin = playerwin.playerwin(screen,
                                             3, rightpanelw, 0, leftpanelw,
                                             config.colors["player"],
                                             self.channel,
                                             self.players[0],
                                             config.keybindings)

        try:
            sp = self.players[1]
        except IndexError:
            sp = None
            
        self.iteminfowin = iteminfowin.iteminfowin(screen,
                                                   6, rightpanelw, 3, leftpanelw,
                                                   config.colors["iteminfo"],
                                                   self.channel,
                                                   sp)

        self.helpwin = helpwin.helpwin(self.screen,
                                       self.h, self.w,
                                       config.colors["help"],
                                       self.channel,
                                       self.timer)
        
        if config.mixer is not None:
            if config.mixerwinpopup:
                from mixerwin import popupmixerwin as mixerwin
            else:
                from mixerwin import statusbarmixerwin as mixerwin
            self.mixerwin = mixerwin(screen,
                                     self.h, self.w,
                                     config.colors["mixer"],
                                     self.channel,
                                     self.timer,
                                     config.mixer,
                                     config.keybindings["global"])
        else:
            del config.keybindings["global"]["volumeup"]
            del config.keybindings["global"]["volumedown"]

        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.activateplaylist, self.activateplaylist)
        self.channel.subscribe(events.activatefilelist, self.activatefilelist)
        self.channel.subscribe(events.playerstart, self.playerstart)

        events.hub.notify(events.activatefilelist())

        curses.curs_set(0)
        curses.meta(1)
        curses.raw()
        curses.halfdelay(1)
        curses.mousemask(curses.ALL_MOUSE_EVENTS)

    def run(self):
        """ main loop of control thread """
        while 1:
            try:
                key = self.screen.getch()
                
                if key==27:
                    # handle escape sequence (e.g. alt+key)
                    key = self.screen.getch()+1024

                if key==curses.KEY_MOUSE:
                    mouse = curses.getmouse()
                    x, y = mouse[1:3]
                    state = mouse[4]
                    events.hub.notify(events.mouseevent(y, x, state))
                elif key==curses.KEY_RESIZE:
                    self.resizeterminal()   
                elif key in self.keybindings["exit"]:
                    curses.halfdelay(5)
                    key = self.screen.getch()
                    curses.halfdelay(1)
                    if key in self.keybindings["exit"]:
                        break
                elif key!=-1:
                    events.hub.notify(events.keypressed(key))


                self.timer.process()
                self.channel.process()

                # update screen
                if key:
                    curses.panel.update_panels()
                    curses.doupdate()

            except KeyboardInterrupt:
                pass

    def quit(self):
        """ cleanup """
        for player in self.players:
            player.playerquit()

        self.songdb.close()

    def getmaxyx(self):
        # taken from http://groups.google.com/groups?selm=uk7xtwasm.fsf%40python.net
        try:
            return int(os.environ["LINES"]), int(os.environ["COLUMNS"])
        except KeyError:
            height, width = struct.unpack(
                "hhhh", fcntl.ioctl(0, termios.TIOCGWINSZ ,"\000"*8))[0:2]
            if not height: return 25, 80
            return height, width

    def refresh(self):
        # refresh screen (i don't know any better way...)
        self.screen.clear()
        self.filelistwin.update()
        self.playlistwin.update()
        self.playerwin.update()
        self.iteminfowin.update()
        self.statusbar.update()

    def resizeterminal(self):
        self.h, self.w = self.getmaxyx()

        leftpanelw = int(self.w/2.2)
        rightpanelw = self.w-leftpanelw

        self.filelistwin.resize(self.h-1, leftpanelw, 0, 0)
        self.playerwin.resize(3, rightpanelw, 0, leftpanelw)
        self.iteminfowin.resize(6, rightpanelw, 3, leftpanelw)
        self.playlistwin.resize(self.h-6-3-1, rightpanelw, 9, leftpanelw)
        self.statusbar.resize(self.h-1, self.w)
        self.helpwin.resize(self.h, self.w)
        self.inputwin.resize(self.h, self.w)
        
        if self.__dict__.has_key("mixerwin"):
            self.mixerwin.resize(self.h, self.w)

        self.filelistwin.update()
        self.playlistwin.update()
        self.playerwin.update()
        self.iteminfowin.update()
        self.statusbar.update()


    # event handler

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["refresh"]:
            self.refresh()
        elif key in self.keybindings["playlistdeleteplayedsongs"]:
            events.hub.notify(events.playlistdeleteplayedsongs())
        elif key in self.keybindings["playlistclear"]:
            events.hub.notify(events.playlistclear())
        elif key in self.keybindings["playlistload"]:
            events.hub.notify(events.playlistload())
        elif key in self.keybindings["playlistsave"]:
            events.hub.notify(events.playlistsave())
        elif key in self.keybindings["showhelp"]:
            if self.filelistwin.hasfocus():
                context = "filelist"
            elif self.playlistwin.hasfocus():
                context = "playlist"
            else:
                context = None

            self.helpwin.showhelp(context)
        else:
            helper.debug("unknown key: %d\n" % key)

    def activateplaylist(self, event):
        self.playlistwin.top()

    def activatefilelist(self, event):
        self.filelistwin.top()

    def playerstart(self, event):
        if event.player is None:
            events.hub.notify(events.playerstart(self.players[0]))

