# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import copy
import time
import threading

import events

STOP     = 0
WANTPLAY = 1
PAUSE    = 3 # not used yet
PLAY     = 4

class playbackinfo:
    
    """ class for storage of playback information """
    
    def __init__(self, player, song=None, time=0, crossfade=0):
        """ 
        player:    player which this playbackinfo instance refers to
        song:      song currently played (or None, if player is not playing)
        time:      position in seconds in the song
        crossfade: crossfade in progress
        """
        self.player = player
        self.song = song
        self.time = time
        self.crossfade = 0

    def __cmp__(self, other):
        return (cmp(self.player, other.player) or
                cmp(self.song, other.song) or
                cmp(self.time, other.time) or
                cmp(self.crossfade, other.crossfade))

    def __str__(self):
        if self.song is not None:
            return "player %s plays %s at time %f)" % (`self.player`, `self.song`, self.time)
        else:
            return "player %s stop" % `self.player`

    __repr__ = __str__

    def updatesong(self, song):
        """ update song and reset time """
        self.song = song
        self.time = 0

    def updatetime(self, time):
        self.time = time

    def updatecrossfade(self, crossfade):
        self.crossfade = crossfade

    def isplaying(self):
        return self.song is not None

    def iscrossfading(self):
        return self.crossfade
        

class genericplayer(threading.Thread):
    def __init__(self, autoplay):
        
        self.autoplay = autoplay
        
        # initialize player state
        if self.autoplay:
            self.state = WANTPLAY
        else:
            self.state = STOP

        self.playbackinfo = playbackinfo(self)

        # old playbackinfo and state , used to detect changes
        self.oplaybackinfo = copy.copy(self.playbackinfo)
        self.ostate = self.state
        
        # have we already requested a new song for crossfading?
        self.autorequestednextsong = 0

        # as independent thread, we want our own event channel
        self.channel = events.hub.newchannel()

        self.done = 0

        threading.Thread.__init__(self)

        # we are only servants of higher powers
        self.setDaemon(1)

        self.channel.subscribe(events.playerstart, self.playerstart)
        self.channel.subscribe(events.playerstop, self.playerstop)
        self.channel.subscribe(events.playerforward, self.playerforward)
        self.channel.subscribe(events.nextsong, self.nextsong)

    def run(self):
        """main loop"""
        while not self.done:
            self.processevents()
            self.play()
            self.updatestatus()

    def processevents(self):
        """check for incoming events and process them"""

        if self.state==WANTPLAY:
            self.requestnextsong()
        if self.state!=PLAY:
            time.sleep(0.5)
                
        # process incoming events
        self.channel.process()

    def play(self):
        """play songs

        has to be provided by specialized classes"""
        pass

    def updatestatus(self):
        """notify interested parties of changes in player status """
        if not self.playbackinfo.isplaying():
            if self.autoplay:
                if self.state!=STOP:
                    self.state = WANTPLAY
            else:
                self.state = STOP
        else:
            self.state = PLAY

        # notify the interested parties, if the player state has changed
        if self.ostate!=self.state or self.oplaybackinfo!=self.playbackinfo:
            self.ostate = self.state
            self.oplaybackinfo = copy.copy(self.playbackinfo)
            
            events.hub.notify(events.playbackinfochanged(self.playbackinfo))

    def playerquit(self):
        """quit player"""
        self.done = 1
        self._playerquit()

    def requestnextsong(self):
        """request next song from playlist, if not already one was requested"""
        if not self.autorequestednextsong:
            self.autorequestednextsong = 1
            events.hub.notify(events.requestnextsong(self))

    def _nextsong(self, song):
        """add song to playlist

        has to be provided by specialized classes"""
        pass

    def _playerstart(self):
        """prepare player fo playing

        has to be provided by specialized classes"""
        pass

    def _playerstop(self):
        """stop playing

        has to be provided by specialized classes"""
        pass
    
    def _playerquit(self):
        """quit player

        has to be provided by specialized classes"""
        pass

    # event handlers

    def playerstart(self, event):
        """start playing"""
        if event.player is self and not self.state==PLAY:
            self._playerstart()
            self.requestnextsong()
            self.state = PLAY

    def playerstop(self, event):
        """stop playing"""
        if event.player is self:
            self._playerstop()
            self.state = STOP

    def playerforward(self, event):
        """immediately play next song"""
        if event.player is self:
            events.hub.notify(events.requestnextsong(self))

    def nextsong(self, event):
        """play event.song next"""
        if event.player is self:
            if event.song:
                self.state = PLAY
                self._nextsong(event.song)
                
            # reset autorequestednextsong, but do this after the
            # self._nextsong call, since this method may infer from
            # self.autorequestednextsong whether a manual (via
            # playerforward) or automatic song change has to be
            # performed
            self.autorequestednextsong = 0
